import greenfoot.*;
import java.util.List;
import java.util.ArrayList;

/**
 * Diese Klasse ist die Basisklasse fr alle Tiere.
 * Zustzlich zu der Standardmethoden aus der Klasse Actor bietet diese Klasse die
 * Mglichkeit zum Bewegen und Drehen an.
 * 
 * @author Michael Kolling
 * @author Gunnar Johannesmeyer (bersetzung und kleinere Anpassung)
 * @version 1.0

 */
public class Object extends Actor
{
    // Konstante
    private static final double BEWEGUNGSGESCHWINDIGKEIT = 5.0;
    
    /**
     * Konstruktor der Klasse Tier - hier wird nichts gemacht.
     */
    public Object()
    {
    }

    /**
     * Act - leere Methode. Tiere haben keine Standartverhalten, was sie automatisch tun.
     * Muss in Unterklassen berschrieben werden.
     */
    public void act()
    {
    }
    
    
    /**
     * Drehe den bergebenen Wert in Grad nach rechts.
     * Das p vor pGrad zeigt an, dass es ein Parameter (bergabewert) ist.
     */
    public void turn(int pGrad)
    {
        setRotation(getRotation() + pGrad);
    }
    

    /**
     * Bewegt Objekt in aktueller Richtung nach vorne.
     */
    public void move()
    {
        double grad = Math.toRadians( getRotation() );
        int x = (int) Math.round(getX() + Math.cos(grad) * BEWEGUNGSGESCHWINDIGKEIT);
        int y = (int) Math.round(getY() + Math.sin(grad) * BEWEGUNGSGESCHWINDIGKEIT);
        
        setLocation(x, y);
    }
    /**
     * Bewegt Objekte in aktueller Richtung nach hinten. (Rckwrtsgang)
     */
    public void backward()
    {
     double grad = Math.toRadians( getRotation() );
        int x = (int) Math.round(getX() + Math.cos(grad) * -BEWEGUNGSGESCHWINDIGKEIT);
        int y = (int) Math.round(getY() + Math.sin(grad) * -BEWEGUNGSGESCHWINDIGKEIT);
        
        setLocation(x, y);
    }
    
    /**
     * Teste, ob sich das Objekt in der Nhe des Randes der Welt befindet.
     * Gebe true (wahr) zurck, falls dies der Fall ist, anderfalls false.
     */
    public boolean amWeltRand()
    {
        if(getX() < 20 || getX() > getWorld().getWidth() - 20)
            return true;
        if(getY() < 20 || getY() > getWorld().getHeight() - 20)
            return true;
        else
            return false;
    }
    
    
    /**
     * Gib true (wahr) zurck, falls wir eine Objekt vom Typ 'pKlassentyp' an der Stelle
     * sehen, wo wir sind. 
     * Gib false (falsch) zurck, wenn hier kein Objekt diesen Typs ist.
     */
    public boolean canSee(Class pKlassentyp)
    {
        Actor akteur = getOneObjectAtOffset(0, 0, pKlassentyp);
        return akteur != null;        
    }

   
    
    /**
     * Versuche, ein Objekt des Typs pKlassentyp zu fressen.
     * Dies ist nur dann mglich, wenn sich an der aktuellen Position ein Objekt des Typs befindet.
     * Anderfalls macht diese Methode nichts.
     */
    public void eat(Class pKlassentyp)
    {
        Actor akteur = getOneObjectAtOffset(0, 0, pKlassentyp);
        if(akteur != null) {
            getWorld().removeObject(akteur);
        }
    }
   
}